﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace FaceDetection.Evolution
{
  public sealed class GeneFactory
  {
    private readonly Dictionary<int, NodeGene> nodes;
    private readonly Dictionary<Tuple<NodeGene, NodeGene>, int> links;
    private readonly List<NodeGene> inputs;
    private readonly List<NodeGene> outputs;

    public GeneFactory(int inputsCount, int outputsCount)
    {
      nodes = new Dictionary<int, NodeGene>();
      links = new Dictionary<Tuple<NodeGene, NodeGene>, int>();
      inputs = new List<NodeGene>(inputsCount);
      outputs = new List<NodeGene>(outputsCount);

      int nodeId = 0;
      for (int i = 0; i < inputsCount; ++i)
      {
        inputs.Add(new NodeGene(--nodeId, NodeType.Input, 0));
      }
      for (int j = 0; j < outputsCount; ++j)
      {
        outputs.Add(new NodeGene(--nodeId, NodeType.Output, 1));
      }
    }

    public List<NodeGene> CreateInitialNodes()
    {
      return inputs.Concat(outputs).ToList();
    }

    public List<LinkGene> CreateInitialLinks()
    {
      return inputs.SelectMany(i => outputs, (i, o) => CreateLink(i, o)).ToList();
    }

    public NodeGene CreateNode(LinkGene link)
    {
      NodeGene node;
      if (!nodes.TryGetValue(link.Id, out node))
      {
        node = new NodeGene(nodes.Count, NodeType.Hidden, (link.From.Depth + link.To.Depth) / 2);
        nodes[link.Id] = node;
      }
      return node;
    }

    public LinkGene CreateLink(NodeGene from, NodeGene to)
    {
      int id;
      var key = Tuple.Create(from, to);
      if (!links.TryGetValue(key, out id))
      {
        id = links.Count;
        links[key] = id;
      }
      return new LinkGene(id, from, to)
      {
        IsEnabled = true,
        IsMutable = true
      };
    }
  }
}
