﻿using System.Collections.Generic;
using System.Linq;
using FaceDetection.Evolution;

namespace FaceDetection.Neural
{
  public sealed class Network
  {
    private readonly int inputsCount;
    private readonly int outputsCount;
    
    private readonly List<Node> nodes;
    private readonly List<Link> links;
    
    public Network(IEnumerable<NodeGene> nodeGenes, IEnumerable<LinkGene> linkGenes)
    {
      inputsCount = 0;
      outputsCount = 0;
      nodes = new List<Node>();
      links = new List<Link>();
      var idNodes = new Dictionary<int, Node>();

      foreach (var gene in nodeGenes.OrderBy(g => g.Depth))
      {
        Node node;
        if (gene.Type == NodeType.Input)
        {
          node = new InputNode();
          ++inputsCount;
        }
        else
        {
          node = new SigmoidNode();
          if (gene.Type == NodeType.Output)
          {
            ++outputsCount;
          }
        }
        idNodes[gene.Id] = node;
        nodes.Add(node);
      }

      foreach (var gene in linkGenes.Where(g => g.IsEnabled))
      {
        idNodes[gene.From.Id].AddLink(idNodes[gene.To.Id], gene.Weight);
      }
    }

    public void SetInputs(IList<double> source)
    {
      for (int i = 0; i < inputsCount; ++i)
      {
        nodes[i].Input = source[i];
      }
    }

    public void CalculateOutputs()
    {
      foreach (var node in nodes)
      {
        node.Activate();
      }
    }

    public IEnumerable<double> GetOutputs()
    {
      for (int i = nodes.Count - outputsCount - 1; i < nodes.Count; ++i)
      {
        yield return nodes[i].Output;
      }
    }
  }
}
